;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Growing Pipe (sprite 49), by imamelia
;;
;; This is a disassembly of sprite 49 in SMW, the growing pipe.
;;
;; Uses first extra bit: YES
;;
;; If the first extra bit is clear, the pipe will start out going up.  If the first extra
;; bit is set, the pipe will start out going down.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

YSpeed:
db $00,$F0,$00,$10

FrameCount:
db $20,$40,$20,$40

TilesToGen1:
db $00,$14,$00,$02

TilesToGen2:
db $00,$15,$00,$02

!PipeTile1 = $A4
!PipeTile2 = $A6

!ExtraBit = $04

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc

LDA #$40
STA $1534,x

LDA $7FAB10,x		;
AND #!ExtraBit		; if the extra bit is set...
BEQ NotUpsideDown	;

INC $C2,x		; make the sprite start out in state 02
INC $C2,x		; so that it goes down instead

NotUpsideDown:		;

RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR GrowingPipeMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GrowingPipeMain:

LDA $1534,x	;
BMI Continue	; if the timer is positive...

LDA $D8,x	;
PHA		;
SEC		;
SBC $1534,x	;
STA $D8,x	; offset the sprite's Y position
LDA $14D4,x	;
PHA		;
SBC #$00		;
STA $14D4,x	;

LDY #$03		;
JSR GenerateTiles	;
PLA		;
STA $14D4,x	;
PLA		;
STA $D8,x	;

LDA $1534,x	;
SEC		;
SBC #$10		;
STA $1534,x	;

RTS

Continue:		;

JSR GrowingPipeGFX

JSR SubOffscreenX0

LDA $9D			; if sprites or locked...
ORA $15A0,x		; or the sprite is offscreen horizontally...
BNE SkipPositionUpdate	; don't update its position

JSR SubHorzPos		;

LDA $0F			;
CLC			;
ADC #$50		; if the sprite is more than 5 tiles away from the player...
CMP #$A0		;
BCS SkipPositionUpdate	; don't update the sprite's position

LDA $C2,x		;
AND #$03		;
TAY			;
INC $1570,x		;
LDA $1570,x		; after a certain number of frames...
CMP FrameCount,y		;
BNE NoChangeState		;
STZ $1570,x		;
INC $C2,x		; change the sprite state
BRA SkipPositionUpdate	;

NoChangeState:		;

LDA YSpeed,y		;
STA $AA,x		; if the sprite Y speed is nonzero...
BEQ SkipTileGen		;

LDA $D8,x		;
AND #$0F		; and the sprite is centered over a tile...
BNE SkipTileGen		;

JSR GenerateTiles		; generate pipe tiles

SkipTileGen:		;

JSL $01801A		; update sprite Y position

SkipPositionUpdate:		;

JSL $01B44F		; make the sprite solid

RTS			;

GenerateTiles:		;

LDA $7FAB10,x		;
AND #!ExtraBit		;
BEQ NotUpsideDown2	;

TYA			;
INC			;
INC			;
AND #$03		;
TAY			;

NotUpsideDown2:		;

LDA TilesToGen1,y		;
STA $185E		; first tile
LDA TilesToGen2,y		;
STA $18B6		; second tile

LDA $185E		;
STA $9C			;
LDA $E4,x		;
STA $9A			; set the position
LDA $14E0,x		;
STA $9B			; of the tile
LDA $D8,x		;
STA $98			; that will be generated
LDA $14D4,x		;
STA $99			;

JSL $00BEB0		; generate Map16 tile routine

LDA $18B6		;
STA $9C			;
LDA $E4,x		;
CLC			;
ADC #$10		; generate the second tile 1 tile to the right
STA $9A			;
LDA $14E0,x		;
ADC #$00		;
STA $9B			;
LDA $D8,x		;
STA $98			;
LDA $14D4,x		;
STA $99			;

JSL $00BEB0		;

RTS			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GrowingPipeGFX:

JSR GetDrawInfo

LDA $00		;
STA $0300,y	;
CLC		;
ADC #$10	;
STA $0304,y	;

LDA $01		;
DEC		;
STA $0301,y	;
STA $0305,y	;

LDA #!PipeTile1	;
STA $0302,y	;
LDA #!PipeTile2	;
STA $0306,y	;

LDA $15F6,x	;
ORA $64		;
STA $0303,y	;
STA $0307,y	;

LDA #$01		;
LDY #$02		;
JSL $81B7B3	;
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS

SubHorzPos:

LDY #$00
LDA $94
SEC
SBC $E4,x
STA $0F
LDA $95
SBC $14E0,x
BPL $01
INY
RTS











